from .dock import Dock, DockArea
from .list_picker import ListPicker
from .file_io import select_file
from .widgets.table import TableView, TableModel, SpinBoxDelegate, BooleanDelegate
from .widgets.push import PushButtonIcon, EditPush, EditPushRel, EditPushInfo
from .widgets.qled import QLED
from .widgets.spinbox import QSpinBox_ro, QSpinBoxWithShortcut
from .custom_app import CustomApp

from .utils import set_dark_palette